# Copyright 2019 Vitaliy Zakaznikov (TestFlows Test Framework http://testflows.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
import subprocess

def transform(file, command, tail=False):
    """Read lines from a file-like object and filter them using
    Unix tail and grep utilities.

    :param file: open file handle
    :pram
    :param tail: tail mode, default: False
    """
    yield None

    process = subprocess.Popen(command, stdin=file, stdout=subprocess.PIPE, shell=True)
    while True:
        line = process.stdout.readline()
        if line == b"":
            if process.poll() is not None:
                break
            else:
                continue
        yield line.decode("utf-8")

